#!/usr/bin/env python

import metcalcs as met_calcs
import metcalc_gui as gui
import sys

if sys.version_info[0] < 3:
    import Tkinter as tk
else:
    import tkinter as tk

__author__ = 'Mark Baker  email: mark.baker@metoffice.gov.uk'


class Model:
    """Perform meteorological calculations calling methods contained within the 'metcalcs' module."""
    def __init__(self):
        self.qnh = ''
        self.qfe = ''
        self.qff = ''
        self.dewpoint_fm_humi = 0.0
        self.dewpoint_fm_wetb = 0.0
        self.humi_fm_wetb = 0.0
        self.fogpt = 0.0
        self.minvis = 0
        self.duskvis = 0
        self.wind1200 = 0
        self.steep_inv = 0
        self.steep_inv_gust = 0
        self.shall_inv = 0
        self.shall_inv_gust = 0
        self.salr = 0
        self.salr_gust = 0
        self.one_c = 0
        self.one_c_gust = 0
        self.dalr_sfc = 0
        self.dalr_sfc_gust = 0
        self.dalr_6000 = 0
        self.dalr_6000_gust = 0
        self.large_cu = 0
        self.large_cu_gust = 0
        self.base = 0
        self.base_alt = 0

    def get_qnh(self, obs_press, temp, baro_ht, elevation, latitude):
        """Get the QNH pressure from observed pressure.
        :param obs_press: The sensor pressure reading in hPa.
        :param temp: The air temperature in degrees C.
        :param baro_ht: The barometer height above site level in metres.
        :param elevation: The altitude of the site above mean sea level in metres.
        :param latitude: The latitude of the site e.g. 52 degrees.
        """
        self.qnh = '{:.2f}'.format(round(met_calcs.calc_qnh_alt(obs_press, temp, elevation, baro_ht), 2))
        self.qfe = '{:.2f}'.format(round(met_calcs.calc_qfe(temp, obs_press, baro_ht), 2))
        self.qff = '{:.2f}'.format(round(met_calcs.calc_qff(temp, float(self.qfe), elevation, latitude), 2))
        self.inhg_flag = True

    def convert_to_inhg(self, qnh_hpa, qfe_hpa, qff_hpa):
        """Convert QNH, QFE, QFF values in hectopascals to inches of mercury
        :param qfe_hpa:
        :param qff_hpa:
        :param qnh_hpa: """
        qnh_inhg = '{:.2f}'.format(round(float(qnh_hpa) * 0.0295299830714, 2))
        qfe_inhg = '{:.2f}'.format(round(float(qfe_hpa) * 0.0295299830714, 2))
        qff_inhg = '{:.2f}'.format(round(float(qff_hpa) * 0.0295299830714, 2))

        return qnh_inhg, qfe_inhg, qff_inhg

    def get_dewpt_fm_humi(self, temp, humidity):
        """Get Dew Point temperature in degrees C from the air temperature and humidity.
        :param temp: Air temperature in degrees C.
        :param humidity: Air humidity %
        """
        self.dewpoint_fm_humi = '{:.1f}'.format(round(met_calcs.calc_dewpoint(humidity, temp), 2))

    def get_dewpt_fm_wetb(self, temp, wetb, pressure):
        """Get Dew Point from the air temperature and wet bulb temperature.
        :param pressure:
        :param temp: Air temperature in degrees C.
        :param wetb: Wet Bulb temperature in degrees C.
        """
        self.dewpoint_fm_wetb = '{:.1f}'.format(round(met_calcs.calc_dewpoint_fm_wetbulb(temp, wetb, pressure), 2))

    def get_humi_fm_wetb(self, temp, wetb, pressure):
        """Get the humidity for a given air temperature and wet bulb temperature.
        :param pressure:
        :param temp: Air temperature in degrees C.
        :param wetb: Wet Bulb temperature in degrees C.
        """
        self.humi_fm_wetb = '{:.0f}'.format(round(met_calcs.calc_humid_fm_wetbulb(temp, wetb, pressure), 0))

    def get_fog(self, mintemp, dewpt, windpsd, maxvis):
        """Get the minimum overnight visibility(M), dusk visibility(M) and fog point as calculated by the modified
            Middle Wallop method.
        :param mintemp: Minimum overnight temperature in degrees C.
        :param dewpt: The dew point temperature in degrees C at time of maximum visibility.
        :param windpsd: Overnight wind speed factor (1 = 0-5kts, 2 = 6-9kts, 3 = >9 kts).
        :param maxvis: Maximum daytime visibility in metres.
        """
        middle_wallop_results = met_calcs.middle_wallop(mintemp, dewpt, windpsd, maxvis)
        self.fogpt = '{:.1f}'.format(round(middle_wallop_results[2], 2))
        self.minvis = '{:.0f}'.format(round(middle_wallop_results[0], -2))
        self.duskvis = '{:.0f}'.format(round(middle_wallop_results[1], -2))

    def get_winds(self, grad_wind):
        """Get the wind speed and max gust expected for given gradient wind speed and pre-set atmospheric stability
            profiles.
        :param grad_wind: Gradient wind speed in kts.
        """
        wind_results = met_calcs.calc_winds(grad_wind)
        self.wind1200 = '{:.0f}'.format(wind_results[0])
        self.steep_inv = wind_results[1]
        self.steep_inv_gust = wind_results[2]
        self.shall_inv = wind_results[3]
        self.shall_inv_gust = wind_results[4]
        self.salr = wind_results[5]
        self.salr_gust = wind_results[6]
        self.one_c = wind_results[7]
        self.one_c_gust = wind_results[8]
        self.dalr_sfc = wind_results[9]
        self.dalr_sfc_gust = wind_results[10]
        self.dalr_6000 = wind_results[11]
        self.dalr_6000_gust = wind_results[12]
        self.large_cu = wind_results[13]
        self.large_cu_gust = wind_results[14]

    def get_base(self, temp, dewpt, land_state):
        """Get the calculated stratus base for a given air temperature, dew point and land warming/colling state.
        :param temp: Air temperature in degrees C.
        :param dewpt: Dew point in degrees C.
        :param land_state: Either 1 = warming land or other integer to indicate cooling land.
        """
        self.base = '{:.0f}'.format(round(met_calcs.calc_stratus_base(temp, dewpt, land_state), 0))

    def get_base_alt(self, sfc_wind, air_state):
        """Get the calculated stratus base for a given surface wind speed in knots and moist or dry air.
        :param air_state: 0 = dry air, 1 - moist air.
        :param sfc_wind: Surface wind speed in knots.
        """
        self.base_alt = '{:.0f}'.format(round(met_calcs.calc_stratus_base_alt(sfc_wind, air_state), 0))


class Controller:
    """Main GUI controller, handles mouse, keyboard and data change events."""

    def __init__(self):
        self.root = tk.Tk()
        self.model = Model()
        self.main_view = gui.MainView(self.root)
        self.inhg_flag = True

        self.main_view.qnh_view.calc_qnh_button.bind('<Button>', self.get_qnh)
        self.main_view.qnh_view.calc_qnh_button.bind('<Return>', self.get_qnh)
        self.main_view.qnh_view.calc_inhg_button.bind('<Button>', self.convert_inhg)
        self.main_view.qnh_view.calc_inhg_button.bind('<Return>', self.convert_inhg)
        self.main_view.dewpt_hum_view.calc_dewpt_button.bind('<Button>', self.get_dewpt_fm_humi)
        self.main_view.dewpt_hum_view.calc_dewpt_button.bind('<Return>', self.get_dewpt_fm_humi)
        self.main_view.dewpt_wetbulb_view.calc_dewpt_button.bind('<Button>', self.get_dewpt_fm_wetb)
        self.main_view.dewpt_wetbulb_view.calc_dewpt_button.bind('<Return>', self.get_dewpt_fm_wetb)

        self.main_view.middlewallop_view.mintemp_scale.bind('<ButtonRelease>', self.get_fog)
        self.main_view.middlewallop_view.dewpt_scale.bind('<ButtonRelease>', self.get_fog)
        self.main_view.middlewallop_view.maxvis_scale.bind('<ButtonRelease>', self.get_fog)

        self.main_view.middlewallop_view.wind5.bind('<ButtonRelease>', self.get_fog1)
        self.main_view.middlewallop_view.wind9.bind('<ButtonRelease>', self.get_fog2)
        self.main_view.middlewallop_view.wind10.bind('<ButtonRelease>', self.get_fog3)

        self.main_view.windgust_view.calc_wind_button.bind('<Button>', self.get_winds)
        self.main_view.windgust_view.calc_wind_button.bind('<Return>', self.get_winds)

        self.main_view.stratus_view.calc_stratus_button.bind('<Button>', self.get_stratus_base)
        self.main_view.stratus_view.calc_stratus_button.bind('<Return>', self.get_stratus_base)

        self.main_view.stratus_view.calc_stratus_button2.bind('<Button>', self.get_stratus_base_alt)
        self.main_view.stratus_view.calc_stratus_button2.bind('<Return>', self.get_stratus_base_alt)

    def run(self):
        """Start the application"""
        self.root.title('Met Calculator')
        self.root.deiconify()
        self.root.mainloop()

    def get_qnh(self, event):
        """Handle response to QNH/QFE/QFF calculate request.
        :param event: QNH/QFE/QFF calculate button or enter.
        """
        self.main_view.qnh_view.qnh.set('ERROR')
        self.main_view.qnh_view.qff.set('ERROR')
        self.main_view.qnh_view.qfe.set('ERROR')

        self.model.get_qnh(self.main_view.qnh_view.obs_press.get(), self.main_view.qnh_view.temp.get(),
                           self.main_view.qnh_view.baro_ht.get(), self.main_view.qnh_view.elevation.get(),
                           self.main_view.qnh_view.latitude.get())

        self.main_view.qnh_view.qnh.set(self.model.qnh)
        self.main_view.qnh_view.qfe.set(self.model.qfe)
        self.main_view.qnh_view.qff.set(self.model.qff)
        self.main_view.qnh_view.qnhlabel.set('QNH (hPa) :')
        self.main_view.qnh_view.qfelabel.set('QFE (hPa) :')
        self.main_view.qnh_view.qfflabel.set('QFF (hPa) :')
        self.inhg_flag = True

    def convert_inhg(self, event):
        if self.inhg_flag:
            pressures_inhg = self.model.convert_to_inhg(self.main_view.qnh_view.qnh.get(),
                                                        self.main_view.qnh_view.qfe.get(),
                                                        self.main_view.qnh_view.qff.get())
            self.main_view.qnh_view.qnh.set(pressures_inhg[0])
            self.main_view.qnh_view.qfe.set(pressures_inhg[1])
            self.main_view.qnh_view.qff.set(pressures_inhg[2])
            self.main_view.qnh_view.qnhlabel.set('QNH (inHg):')
            self.main_view.qnh_view.qfelabel.set('QFE (inHg):')
            self.main_view.qnh_view.qfflabel.set('QFF (inHg):')
            self.inhg_flag = False

    def get_dewpt_fm_humi(self, event):
        """Handle response to dew point from humidity calculate request.
        :param event: Mouse button or enter pressed on calculate button.
        """
        self.main_view.dewpt_hum_view.result_dewpt.set('ERROR')
        self.model.get_dewpt_fm_humi(self.main_view.dewpt_hum_view.temp.get(),
                                     self.main_view.dewpt_hum_view.humidity.get())

        self.main_view.dewpt_hum_view.result_dewpt.set(self.model.dewpoint_fm_humi)

    def get_dewpt_fm_wetb(self, event):
        """Handle response to dew point from wet bulb calculate request.
        :param event: Mouse button or enter pressed on calculate button.
        """
        self.main_view.dewpt_wetbulb_view.result_dewpt.set('ERROR')
        self.main_view.dewpt_wetbulb_view.result_humi.set('ERROR')

        self.model.get_dewpt_fm_wetb(self.main_view.dewpt_wetbulb_view.temp.get(),
                                     self.main_view.dewpt_wetbulb_view.wetbulb.get(),
                                     self.main_view.dewpt_wetbulb_view.pressure.get())

        self.model.get_humi_fm_wetb(self.main_view.dewpt_wetbulb_view.temp.get(),
                                    self.main_view.dewpt_wetbulb_view.wetbulb.get(),
                                    self.main_view.dewpt_wetbulb_view.pressure.get())

        self.main_view.dewpt_wetbulb_view.result_dewpt.set(self.model.dewpoint_fm_wetb)
        self.main_view.dewpt_wetbulb_view.result_humi.set(self.model.humi_fm_wetb)

    def get_fog(self, event):
        """Handle response to fog/vis slider change recalculate request.
        :param event: Fog/Vis method slider changed.
        """
        self.model.get_fog(self.main_view.middlewallop_view.mintemp_scale.get(),
                           self.main_view.middlewallop_view.dewpt_scale.get(),
                           self.main_view.middlewallop_view.wind.get(),
                           self.main_view.middlewallop_view.maxvis_scale.get())

        self.main_view.middlewallop_view.result_fogpt.set(self.model.fogpt)
        self.main_view.middlewallop_view.result_minvis.set(self.model.minvis)
        self.main_view.middlewallop_view.result_duskvis.set(self.model.duskvis)

    def get_fog1(self, event):  # Must be an easier way to do this? Tk variables events?
        """Handle response to wind 0-5kts selected calculate request.
        :param event: 0-5 kts radio button selected.
        """
        self.model.get_fog(self.main_view.middlewallop_view.mintemp_scale.get(),
                           self.main_view.middlewallop_view.dewpt_scale.get(),
                           1,
                           self.main_view.middlewallop_view.maxvis_scale.get())

        self.main_view.middlewallop_view.result_fogpt.set(self.model.fogpt)
        self.main_view.middlewallop_view.result_minvis.set(self.model.minvis)
        self.main_view.middlewallop_view.result_duskvis.set(self.model.duskvis)

    def get_fog2(self, event):
        """Handle response to wind 6-9kts selected calculate request.
        :param event: 6-9 kts radio button selected."""
        self.model.get_fog(self.main_view.middlewallop_view.mintemp_scale.get(),
                           self.main_view.middlewallop_view.dewpt_scale.get(),
                           2,
                           self.main_view.middlewallop_view.maxvis_scale.get())

        self.main_view.middlewallop_view.result_fogpt.set(self.model.fogpt)
        self.main_view.middlewallop_view.result_minvis.set(self.model.minvis)
        self.main_view.middlewallop_view.result_duskvis.set(self.model.duskvis)

    def get_fog3(self, event):
        """Handle response to wind 6-9kts selected calculate request.
        :param event: 6-9 kts radio button selected."""
        self.model.get_fog(self.main_view.middlewallop_view.mintemp_scale.get(),
                           self.main_view.middlewallop_view.dewpt_scale.get(),
                           3,
                           self.main_view.middlewallop_view.maxvis_scale.get())

        self.main_view.middlewallop_view.result_fogpt.set(self.model.fogpt)
        self.main_view.middlewallop_view.result_minvis.set(self.model.minvis)
        self.main_view.middlewallop_view.result_duskvis.set(self.model.duskvis)

    def get_winds(self, event):
        """Handle response to mouse button or enter key on winds calculate
        :param event: Mouse button or enter button pressed on calculate winds.
        """
        self.model.get_winds(self.main_view.windgust_view.gradwind.get())
        self.main_view.windgust_view.result_wind1200.set(self.model.wind1200)

        for i in self.main_view.windgust_view.treeview.get_children():
            self.main_view.windgust_view.treeview.delete(i)

        self.main_view.windgust_view.treeview.insert("", 0, text="Steep inversion",
                                                     values=(int(self.model.steep_inv), int(self.model.steep_inv_gust)))
        self.main_view.windgust_view.treeview.insert("", 1, text="Shallow inversion",
                                                     values=(int(self.model.shall_inv), int(self.model.shall_inv_gust)))
        self.main_view.windgust_view.treeview.insert("", 2, text="SALR from surface",
                                                     values=(int(self.model.salr), int(self.model.salr_gust)))
        self.main_view.windgust_view.treeview.insert("", 3, text="1-2 deg C / 1000ft",
                                                     values=(int(self.model.one_c), int(self.model.one_c_gust)))
        self.main_view.windgust_view.treeview.insert("", 4, text="DALR from surface",
                                                     values=(int(self.model.dalr_sfc), int(self.model.dalr_sfc_gust)))
        self.main_view.windgust_view.treeview.insert("", 5, text="DALR to 6000ft",
                                                     values=(int(self.model.dalr_6000), int(self.model.dalr_6000_gust)))
        self.main_view.windgust_view.treeview.insert("", 6, text="Large CU / Steep LR",
                                                     values=(int(self.model.large_cu), int(self.model.large_cu_gust)))

    def get_stratus_base(self, event):
        """Handle events for stratus base calculator.
        :param event: Mouse button or enter pressed on stratus base calculate.
        """
        self.main_view.stratus_view.result_stratusbase.set('ERROR')
        self.model.get_base(self.main_view.stratus_view.temp.get(), self.main_view.stratus_view.dewpt.get(),
                            self.main_view.stratus_view.land_state.get())

        self.main_view.stratus_view.result_stratusbase.set(self.model.base)

    def get_stratus_base_alt(self, event):
        """Handle events for the alternative stratus base calculator.
        :param event: Mouse button or enter pressed on alternative stratus base calculate.
        """
        self.main_view.stratus_view.result_stratusbase2.set('ERROR')
        self.model.get_base_alt(self.main_view.stratus_view.wind.get(), self.main_view.stratus_view.air_state.get())

        self.main_view.stratus_view.result_stratusbase2.set(self.model.base_alt)


if __name__ == '__main__':
    controller = Controller()
    controller.run()



import sys
import metcalculator

def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]

    controller = metcalculator.Controller()
    controller.run()

    # Do argument parsing here (eg. with argparse) and anything else
    # you want your project to do.

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import math

__author__ = 'Mark Baker - mark2182@mac.com'

""" Python code for calculating various meteorological parameters
by Mark Baker (c)2015 - Mark Baker mark2182@mac.com')"""


def celsius_to_kelvin(temp_c):
    """Convert celsius temperature to Kelvin by adding 273.15.
    :param temp_c: Temperature in degrees C.
    :return temperature in Kelvin
    """
    return temp_c + 273.15


def kelvin_to_celsius(temp_k):
    """Convert Kelvin temperature to Celsius by subtracting 273.15.
    :param temp_k: Temperature in Kelvin.
    :return Temperature in degrees Celsius.
    """
    return temp_k - 273.15


def celsius_to_farenh(temp_c):
    """Convert Celsius temperature to Fahrenheit.
    :param temp_c: Temperature in degees C
    :return Temperature in degrees Fahrenheit.
    """
    return ((temp_c * 9.0) / 5.0) + 32.0


def farenh_to_celsius(temp_f):
    """Convert Fahrenheit temperature to Celsius.
    :param temp_f: Temperature in Fahrenheit.
    :return Temperature in Celsius.
    """
    return ((temp_f - 32.0) * 5.0) / 9.0


def calc_dewpoint(rel_humi, temp_c):
    """Calculate the air mass dewpoint for a given relative humidity (%) and temperature (C)
       using Vaisala's method.

        pws= Saturation vapour pressure (hPa)   A, m, Tn= constants   T= temp (C)
        A = 6.116441  m = 7.591386  Tn = 240.7263
        :param temp_c: Temperature in degrees C.
        :param rel_humi: Percentage relative humidity
        """
    if rel_humi > 0:
        x = (7.591386 * temp_c) / (temp_c + 240.7263)
        pws = 6.116441 * math.pow(10, x)
        pw = pws * (rel_humi / 100.0)
        y = math.log10(pw / 6.116441)
        z = (7.591386 / y) - 1
        dewpoint = 240.7263 / z
        return dewpoint


def calc_dewpoint_fm_wetbulb(temp_c, wetbulb_c, pressure):
    """Calculate the air mass dewpoint for a given dry bulb and wet bulb temperature (C) using
       Vaisala's method.
       :param pressure:
       :param wetbulb_c: Wet bulb temperature in degrees C.
       :param temp_c: Dry bulb temperature in degrees C.
       """

    xdry = (7.591386 * temp_c) / (temp_c + 240.7263)
    xwet = (7.591386 * wetbulb_c) / (wetbulb_c + 240.7263)

    pwswet = 6.116441 * math.pow(10, xwet)

    pw = pwswet - pressure * 0.0008 * (temp_c - wetbulb_c)

    y = math.log10(pw / 6.116441)

    z = (7.591386 / y) - 1

    dewpoint = 240.7263 / z
    return dewpoint


def calc_humid_fm_wetbulb(temp_c, wetbulb_c, pressure):
    """Calculate the air mass humidity (%) for a given dry bulb and wet bulb temperature (C) using
       Vaisala's method.
       :param pressure:
       :param wetbulb_c: Wet bulb temperature in degrees C.
       :param temp_c: Dry bulb temperature in degrees C.
       """
    xdry = (7.591386 * temp_c) / (temp_c + 240.7263)
    xwet = (7.591386 * wetbulb_c) / (wetbulb_c + 240.7263)

    pwsdry = 6.116441 * math.pow(10, xdry)
    pwswet = 6.116441 * math.pow(10, xwet)

    pw = pwswet - pressure * 0.0008 * (temp_c - wetbulb_c)

    rh = (pw / pwsdry) * 100
    return rh


def calc_qfe(temp_c, sensor_pressure, sensor_height):
    """Calculate pressure at site ground level given observed pressure, temperature and height
       above ground of the sensor.

        Applies the 'hypsometric equation':
        QFE = p x (1+ (hQFE x g) / (R x T))
        p = sensor pressure, hQFE = barometer height above station elevation, R = gas const.
        T = temperature in deg C.
        :param sensor_height: Height of barometer above ground level in metres.
        :param sensor_pressure: Pressure reading from sensor in hPa.
        :param temp_c: Temperature in degrees C.
        """

    qfe = sensor_pressure * (1 + ((sensor_height * 9.80665) / (287.04 * (temp_c + 273.15))))
    return qfe


def calc_qnh(qfe, site_elevation):
    """Calculate the mean sea level pressure given QFE pressure, temperature and height
       above sea level (altitude) - ICAO recommended calculation of QNH from QFE see ICAO doc 9837.

        heightISA = 44330.77 - 11880.32 x QFE^0.190263
        QNH = 1013.25 x ((1 - (0.0065 x (heightISA-height)/288.15))^5.25588)
        :param site_elevation: Elevation of the site above mean sea level (MSL).
        :param qfe: QFE pressure reading in hPa."""

    qnh = 1013.23 * (math.pow((1 - ((((44330.77 - (11880.32 * (math.pow(qfe, 0.190263)))) -
                                      site_elevation) / 288.15) * 0.0065)), 5.25588))
    return qnh


def calc_qnh_alt(pressure, temperature, afht, barht):
    """Alternative method for calculating QNH base on Ross Provans (Met Office) spreadsheet
    :param barht: Barometer height above ground level.
    :param afht: Airfield height above mean sea level.
    :param temperature: Observed temperature (deg C).
    :param pressure: Observed pressure (hPa - read from sensor).
    """

    const = (1 + (9.6 * ((math.pow(10, -5)) * afht) + (6 * (math.pow(10, -9)) * (math.pow(afht, 2)))))
    qnh = pressure + ((0.022857 * afht) + ((const - 1) * pressure) + (const * (pressure * ((math.pow(10,
        (barht / (18429.1 + 67.53 * temperature + (0.003 * barht)))))) - pressure)))

    return qnh


def calc_qff(temp_c, qfe, site_elevation, latitude):
    """Calculate the sea level pressure using the Swedish HMI method for automatic stations.

        QFF = QFE x e^(stationElev x (0.034163(1-0.0026373 cos(2w)/t)
        if temp < -7degC  t = 0.5 x temp + 275
                   -7degC <= temp < +2degC   t = 0.54 x temp + 275.6
                     temp > +2degC  t = 1.07 x temp + 274.5
              t is a way of describing the winter inversions.
              :param latitude: Latitude of site (e.g. 52.0).
              :param site_elevation: Site elevation above mean sea level in metres.
              :param qfe: Calculated QFE pressure in hPa.
              :param temp_c: Temperature in degrees C.
              """

    if temp_c < -7:
        t = 0.5 * temp_c + 275
    elif -7 <= temp_c <= 2:
        t = 0.533 * temp_c + 275.6
    else:
        t = 1.07 * temp_c + 274.5

    qff = qfe * (math.pow(2.71828182, ((site_elevation * (0.034163 *
                                (1 - (0.0026373 * (math.cos(2 * latitude)))))) / t)))
    return qff


def middle_wallop(temp_min, dewpt, windfactor, maxvis):
    """Calculate fog point, minimum overnight visibility and dusk visibility using the Middle Wallop
        method as modified by matt.woods@metoffice.gov.uk, fog point calculated using method developed by
        dan.harris@metoffice.gov.uk . This empirical technique takes account of overnight wind, daytime maximum
        visibility and daytime dew point. For further details refer to the Forecasters Reference book visibility
        section.
    :param temp_min: Minimum temperature overnight in degrees C.
    :param dewpt: Dew point at maximum daytime visibility in degrees C.
    :param windfactor: 1 = 0-5kts, 2 = 6-9kts, 3 = >9kts.
    :param maxvis: Maximum daytime visibility in metres.
    :return: A tuple containing minimum visibility(M), dusk visibility(M) and fog point(deg C).
    """
    duskvis = maxvis * 0.8

    if windfactor == 1:
        fogpt = round(dewpt - (math.log(maxvis / 3000) / math.log(2)), 1)
        minvis = int(maxvis * 0.33)

    elif windfactor == 2:
        fogpt = round(dewpt - (math.log(maxvis / 2000) / math.log(2)), 1)
        minvis = int(maxvis * 0.5)

    else:
        fogpt = round(dewpt - (math.log((3 * maxvis) / 4000) / math.log(2)), 1)
        minvis = int(maxvis * 0.75)

    if temp_min - dewpt == 0:
        minvis = minvis
    if temp_min - dewpt < 0:
        minvis /= 2
    if temp_min - dewpt < -1:
        minvis /= 2
    if temp_min - dewpt < -2:
        minvis /= 2
    if temp_min - dewpt < -3:
        minvis /= 2
    if temp_min - dewpt < -4:
        minvis /= 2
    if temp_min - dewpt < -5:
        minvis /= 2
    if temp_min - dewpt < -6:
        minvis /= 2
    if temp_min - dewpt < -7:
        minvis /= 2
    elif temp_min - dewpt > 0:
        minvis = duskvis

    return minvis, duskvis, fogpt


def calc_winds(grad_wind):
    """Calculate the surface mean wind speed and maximum gust expected for a given gradient wind speed. These
     figures are calculated using the figures supplied in the revised forecaster reference book, originally determined
     at RAF Honnington.

    :param grad_wind: The gradient wind in knots.
    :return: A tuple containing the wind speed and gust results for different stability profiles:
    0 - 1200ft wind
    2 - Steep inversion - mean
    2 - Steep inversion - gust
    3 - Shallow inversion - mean
    4 - Shallow inversion - gust
    5 - SALR from surface - mean
    6 - SALR from surface - gust
    7 - 1-2 deg C / 1000ft - mean
    8 - 1-2 deg C / 1000ft - gust
    9 - DALR from surface - mean
    10 - DALR from surface - gust
    11 - DALR to 6000ft - mean
    12 - DALR to 6000ft - gust
    13 - Large CU / Steep LR - mean
    14 - Large CU / Steep LR - gust
    """
    wind1200 = round(grad_wind * 0.9, 0)
    steep_inv = round(grad_wind * (1.0 / 6.0), 0)
    steep_inv_gust = steep_inv
    shallow_inv = round(grad_wind * (3.0 / 10.0), 0)
    shallow_inv_gust = round(shallow_inv * (30.0 / 25.0), 0)
    salr_sfc = round(grad_wind * (2.0 / 5.0), 0)
    salr_sfc_gust = round(salr_sfc * (35.0 / 25.0), 0)
    one_c1000 = round(grad_wind * (25.0 / 49.0), 0)
    one_c1000_gust = round(one_c1000 * (35.0 / 25.0), 0)
    dalr_sfc = round(grad_wind * (25.0 / 45.0), 0)
    dalr_sfc_gust = round(dalr_sfc * (24.5 / 15.0), 0)
    dalr_6000 = round(grad_wind * (30.0 / 49.5), 0)
    dalr_6000_gust = round(dalr_6000 * (24.5 / 15.0), 0)
    large_cu = round(grad_wind * (27.5 / 40.0), 0)
    large_cu_gust = round(large_cu * (24.5 / 15.0), 0)

    return wind1200, steep_inv, steep_inv_gust, shallow_inv, shallow_inv_gust, salr_sfc, salr_sfc_gust, one_c1000, \
           one_c1000_gust, dalr_sfc, dalr_sfc_gust, dalr_6000, dalr_6000_gust, large_cu, large_cu_gust


def calc_stratus_base(temp, dewpt, land_state):
    """Calculate the height of the base of stratus clouds base on the dewpoint and temperature of the air. The
    Base height in feet = z(T-Td) where z is a constant dependant on the land state: 350 = land warming
    450 = land cooling.
    :param temp: Air temperature in degrees C.
    :param dewpt: Dew point in degrees C.
    :param land_state: Land state where 1 = land warming otherwise land assumed to be cooling.
    :return: The height of the base of stratus cloud in feet.
    """
    if land_state is 1:
        factor = 350.0  # land warming
    else:
        factor = 450.0

    base = factor * (temp - dewpt)
    return base


def calc_stratus_base_alt(sfc_wind, air_state):
    """Calculate the height of the base of stratus clouds base on the surface wind and moisture of the air. The
    Base height in feet = surface wind * z where z is a constant dependant on the air state: 75 = moist air
    200 = dry air.
    :param sfc_wind: Surface wind speed in knots.
    :param air_state: 0 = dry air, 1 = moist air.
    :return: The height of the base of stratus cloud in feet.
    """
    if air_state is 1:
        factor = 75.0  # moist air
    else:
        factor = 200.0  # dry air

    base = factor * sfc_wind
    return base


def tests():
    """Perform tests of this modules functions."""

    print('Humid=50%, TempC=20.0 Dewpoint = ', calc_dewpoint(50, 20.0), 'C')
    print('WetbulbC=38.5, TempC=40.0, Pressure=1013: Dewpoint from wetbulb = ', calc_dewpoint_fm_wetbulb(40.0, 38.5, 1013), 'C')
    print('WetbulbC=38.5, TempC=40.0, Pressure=1013 Humidity from wetbulb = ', calc_humid_fm_wetbulb(40.0, 38.5, 1013), '%')
    print('0 deg C = ', celsius_to_kelvin(0), 'K')
    print('0 K = ', kelvin_to_celsius(0), 'C')
    print('0 deg C  = ', celsius_to_farenh(0), 'F')
    print('32 deg F  = ', farenh_to_celsius(32), 'C')
    print('1008.5 hPa, temp=25 C, sensor ht=3 M : QFE= ', calc_qfe(25, 1008.5, 3.0))
    print('QFE=1008.85 hPa, Site elevation=150 M : QNH= ', calc_qnh(1008.85, 150))
    print('QFE=1008.85 hPa, Site elevation=150 M, Temp=25 C : QFF= ', calc_qff(25, 1008.85, 150, 52))
    print('1008.5 hPa, temp=25 C, Site elevation=150 M, Sensor ht=3 M : (alt) QNH= ', calc_qnh_alt(1008.5, 25, 150, 3))
    print('Middle Wallop (Tmin=1C DewPt=4C Windfactor=1 Vismax=15000m - Min vis, Dusk vis, Fog point: '
          , middle_wallop(1, 4, 1, 15000))
    print('Surface wind / gust(kts) - gradient wind 25kts: ', calc_winds(25))
    print('Stratus base(ft) - Temp 8C, Dewpt 7C, Land warming - ', calc_stratus_base(8, 7.5, 1))


if __name__ == '__main__':
    tests()



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='metcalculator',
    version='1.4.5',
    description='Calculate various meteorological parameters',
    url='www.metpod.co.uk',
    license='MIT',
    author='Mark Baker',
    author_email='mark2182@mac.com',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    entry_points={
          'gui_scripts': [
               'metcalculator = metcalculator.__main__:main'
          ]
    },
)



#!/usr/bin/env python
import sys

if sys.version_info[0] < 3:
    import Tkinter as tk
    import ttk
else:
    import tkinter as tk
    from tkinter import ttk as ttk

__author__ = 'Mark Baker  email: mark2182@mac.com'


class MainView:
    """Main GUI class responsible for initiating separate window frames for the various calculation tools."""

    def __init__(self, master):
        self.frame = ttk.Frame(master)
        self.frame.grid(row=0, column=0, padx=5, pady=5)

        # s = ttk.Style()
        # s.theme_use('aqua')

        """self.menubar = tk.Menu(master)
        self.filemenu = tk.Menu(self.menubar, tearoff=0)
        self.filemenu.add_command(label='Quit', command=sys.exit)
        self.menubar.add_cascade(label='MetCalculator', menu=self.filemenu)
        master.config(menu=self.menubar)"""

        self.qnh_view = QNHview(master)
        self.dewpt_hum_view = DewpointFromHumidityView(master)
        self.dewpt_wetbulb_view = DewpointFromWetbulbView(master)
        self.middlewallop_view = MiddleWallop(master)
        self.windgust_view = WindGust(master)
        self.stratus_view = StratusBase(master)



class QNHview:
    """Label window frame containing the widgets for calculating QNH/QFE/QFF. """

    def __init__(self, root):
        validcmd_positive = (root.register(validate_float_positive), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        validcmd = (root.register(validate_float_negative), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        self.frame_qnh = tk.LabelFrame(root, text='QNH QFE QFF')
        self.frame_qnh.grid(row=0, column=0, columnspan=2, sticky=tk.NSEW, padx=5, pady=5)

        self.obs_press_label = ttk.Label(self.frame_qnh, text="Observed Pressure (hPa):")
        self.obs_press_label.grid(sticky=tk.E, row=0, column=0, padx=5, pady=5)

        self.obs_press = tk.DoubleVar()
        self.obs_press_entry = ttk.Entry(self.frame_qnh, textvariable=self.obs_press, width=8, validate='key',
                                         validatecommand=validcmd_positive)
        self.obs_press_entry.grid(row=0, column=2, pady=5)
        self.obs_press.set(1013.25)

        self.temp_label = ttk.Label(self.frame_qnh, text="Temperature (deg C):")
        self.temp_label.grid(sticky=tk.E, row=1, column=0, padx=5, pady=5)

        self.temp = tk.DoubleVar()
        self.temp_entry = ttk.Entry(self.frame_qnh, textvariable=self.temp, width=8, validate='key',
                                    validatecommand=validcmd)
        self.temp_entry.grid(row=1, column=2, pady=5)
        self.temp.set(15.0)

        self.baro_ht_label = ttk.Label(self.frame_qnh, text="Barometer height above ground (metres):")
        self.baro_ht_label.grid(sticky=tk.E, row=2, column=0, padx=5, pady=5)

        self.baro_ht = tk.DoubleVar()
        self.baro_ht_entry = ttk.Entry(self.frame_qnh, textvariable=self.baro_ht, width=8, validate='key',
                                       validatecommand=validcmd)
        self.baro_ht_entry.grid(row=2, column=2, pady=5)
        self.baro_ht.set(4.65)

        self.elevation_label = ttk.Label(self.frame_qnh, text="Station Elevation AMSL (metres):")
        self.elevation_label.grid(sticky=tk.E, row=3, column=0, padx=5, pady=5)

        self.elevation = tk.DoubleVar()
        self.elevation_entry = ttk.Entry(self.frame_qnh, textvariable=self.elevation, width=8, validate='key',
                                         validatecommand=validcmd)
        self.elevation_entry.grid(row=3, column=2, pady=5)
        self.elevation.set(86.35)

        self.latitude_label = ttk.Label(self.frame_qnh, text="Station Latitude (only affects QFF):")
        self.latitude_label.grid(row=4, column=0, padx=5, pady=5, sticky=tk.E)

        self.latitude = tk.DoubleVar()
        self.latitude_entry = ttk.Entry(self.frame_qnh, textvariable=self.latitude, width=8, validate='key',
                                        validatecommand=validcmd_positive)
        self.latitude_entry.grid(row=4, column=2, pady=5)

        self.calc_qnh_button = ttk.Button(self.frame_qnh, text='Calculate')
        self.calc_qnh_button.grid(row=3, column=5, sticky=tk.W, padx=10)

        self.calc_inhg_button = ttk.Button(self.frame_qnh, text='>> inHg')
        self.calc_inhg_button.grid(row=3, column=6, sticky=tk.W, padx=10)

        self.qnhlabel = tk.StringVar()
        self.qnhlabel.set('QNH (hPa) :')
        self.qnh = tk.StringVar()
        self.qnh_result_label = ttk.Label(self.frame_qnh, textvariable=self.qnhlabel)
        self.qnh_result_label.grid(row=0, column=5, sticky=tk.E, padx=5)
        self.qnh_result = ttk.Label(self.frame_qnh, textvariable=self.qnh, width=7)
        self.qnh_result.grid(sticky=tk.W, row=0, column=6, padx=5)

        self.qfelabel = tk.StringVar()
        self.qfelabel.set('QFE (hPa) :')
        self.qfe = tk.StringVar()
        self.qfe_result_label = ttk.Label(self.frame_qnh, textvariable=self.qfelabel)
        self.qfe_result_label.grid(row=1, column=5, sticky=tk.E, padx=5)
        self.qfe_result = ttk.Label(self.frame_qnh, textvariable=self.qfe, width=7)
        self.qfe_result.grid(sticky=tk.W, row=1, column=6, padx=5)

        self.qfflabel = tk.StringVar()
        self.qfflabel.set('QFF (hPa) :')
        self.qff = tk.StringVar()
        self.qff_result_label = ttk.Label(self.frame_qnh, textvariable=self.qfflabel)
        self.qff_result_label.grid(row=2, column=5, sticky=tk.E, padx=5)
        self.qff_result = ttk.Label(self.frame_qnh, textvariable=self.qff, width=7)
        self.qff_result.grid(sticky=tk.W, row=2, column=6, padx=5)


class DewpointFromHumidityView:
    """Label window frame containing the widgets for calculating dew point from humidity. """

    def __init__(self, root):
        validcmd = (root.register(validate_float_negative), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        validcmd_positive = (root.register(validate_float_positive), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        self.frame_dewpt_humi = tk.LabelFrame(root, text='Dew Point from Humidity')
        self.frame_dewpt_humi.grid(row=1, column=0, sticky=tk.NSEW, padx=5, pady=5)

        self.temp_label = ttk.Label(self.frame_dewpt_humi, text="Temperature (deg C):")
        self.temp_label.grid(sticky=tk.E, row=0, column=0, padx=5, pady=5)
        self.temp = tk.DoubleVar()
        self.temp_entry = ttk.Entry(self.frame_dewpt_humi, textvariable=self.temp, width=6, validate='key',
                                    validatecommand=validcmd)
        self.temp_entry.grid(row=0, column=1, pady=5)
        self.temp.set(15.0)

        self.humi_label = ttk.Label(self.frame_dewpt_humi, text="Humidity (%):")
        self.humi_label.grid(sticky=tk.E, row=1, column=0, padx=5, pady=5)
        self.humidity = tk.DoubleVar()
        self.humi_entry = ttk.Entry(self.frame_dewpt_humi, textvariable=self.humidity, width=6, validate='key',
                                    validatecommand=validcmd_positive)
        self.humi_entry.grid(row=1, column=1, pady=5)
        self.humidity.set(75)

        self.dewpt_label = ttk.Label(self.frame_dewpt_humi, text="Dew Pt (deg C):")
        self.dewpt_label.grid(sticky=tk.E, row=3, column=0, padx=5, pady=5)
        self.result_dewpt = tk.StringVar()
        self.result_label = ttk.Label(self.frame_dewpt_humi, textvariable=self.result_dewpt, width=7)
        self.result_label.grid(row=3, column=1, sticky=tk.E, padx=5, pady=5)

        self.calc_dewpt_button = ttk.Button(self.frame_dewpt_humi, text='Calculate')
        self.calc_dewpt_button.grid(row=2, column=0, columnspan=1, sticky=tk.E, )


class DewpointFromWetbulbView:
    """Label window frame containing the widgets for calculating dew point from wet bulb."""

    def __init__(self, root):
        validcmd = (root.register(validate_float_negative), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        self.frame_dewpt_wetb = tk.LabelFrame(root, text='Dew Point from Wet Bulb')
        self.frame_dewpt_wetb.grid(row=1, column=1, sticky=tk.NSEW, padx=5, pady=5)

        self.temp_label = ttk.Label(self.frame_dewpt_wetb, text="Temperature (deg C):")
        self.temp_label.grid(sticky=tk.E, row=0, column=0, padx=5, pady=5)
        self.temp = tk.DoubleVar()
        self.temp_entry = ttk.Entry(self.frame_dewpt_wetb, textvariable=self.temp, width=7, validate='key',
                                    validatecommand=validcmd)
        self.temp_entry.grid(row=0, column=1, pady=5)
        self.temp.set(15.0)

        self.wetb_label = ttk.Label(self.frame_dewpt_wetb, text="Wet Bulb (deg C):")
        self.wetb_label.grid(sticky=tk.E, row=1, column=0, padx=5, pady=5)
        self.wetbulb = tk.DoubleVar()
        self.wetb_entry = ttk.Entry(self.frame_dewpt_wetb, textvariable=self.wetbulb, width=7, validate='key',
                                    validatecommand=validcmd)
        self.wetb_entry.grid(row=1, column=1, pady=5)
        self.wetbulb.set(12.7)

        self.pressure_label = ttk.Label(self.frame_dewpt_wetb, text="Pressure (hPa):")
        self.pressure_label.grid(sticky=tk.E, row=2, column=0, padx=5, pady=5)
        self.pressure = tk.DoubleVar()
        self.pressure_entry = ttk.Entry(self.frame_dewpt_wetb, textvariable=self.pressure, width=7, validate='key',
                                    validatecommand=validcmd)
        self.pressure_entry.grid(row=2, column=1, pady=5)
        self.pressure.set(1013.25)

        self.dewpt_label = ttk.Label(self.frame_dewpt_wetb, text="Dew Pt (deg C):")
        self.dewpt_label.grid(sticky=tk.E, row=4, column=0, padx=5, pady=5)
        self.result_dewpt = tk.StringVar()
        self.result_dewpt_label = ttk.Label(self.frame_dewpt_wetb, textvariable=self.result_dewpt,
                                            width=7)
        self.result_dewpt_label.grid(row=4, column=1, sticky=tk.E, padx=5, pady=5)

        self.humi_label = ttk.Label(self.frame_dewpt_wetb, text="Humidity (%):")
        self.humi_label.grid(sticky=tk.E, row=5, column=0, padx=5, pady=5)
        self.result_humi = tk.StringVar()
        self.result_humi_label = ttk.Label(self.frame_dewpt_wetb, textvariable=self.result_humi,
                                           width=7)
        self.result_humi_label.grid(row=5, column=1, sticky=tk.E, padx=5, pady=5)

        self.calc_dewpt_button = ttk.Button(self.frame_dewpt_wetb, text='Calculate')
        self.calc_dewpt_button.grid(row=3, column=0, columnspan=1, sticky=tk.E, )


class MiddleWallop:
    """Label window frame containing the widgets for calculating fog and visibility parameters
        via Middle Wallop method."""

    def __init__(self, root):
        self.frame_middlewallop = tk.LabelFrame(root, text='Middle Wallop Method (Vis/Fog)')
        self.frame_middlewallop.grid(row=0, column=2, rowspan=2, sticky=tk.NSEW, padx=5, pady=5)

        self.fogpt_label = ttk.Label(self.frame_middlewallop, text="Fog Pt (C):")
        self.fogpt_label.grid(sticky=tk.W, row=0, column=0, padx=10, pady=5)
        self.result_fogpt = tk.StringVar()
        self.result_fogpt_label = ttk.Label(self.frame_middlewallop, textvariable=self.result_fogpt,
                                            width=7)
        self.result_fogpt_label.grid(row=0, column=1, sticky=tk.E, padx=10, pady=5)

        self.minvis_label = ttk.Label(self.frame_middlewallop, text='Min VIS (M):')
        self.minvis_label.grid(sticky=tk.W, row=1, column=0, padx=10, pady=5)
        self.result_minvis = tk.StringVar()
        self.result_minvis_label = ttk.Label(self.frame_middlewallop, textvariable=self.result_minvis,
                                             width=7)
        self.result_minvis_label.grid(row=1, column=1, sticky=tk.E, padx=10, pady=5)

        self.duskvis_label = ttk.Label(self.frame_middlewallop, text='Dusk VIS (M):')
        self.duskvis_label.grid(sticky=tk.W, row=2, column=0, padx=10)
        self.result_duskvis = tk.StringVar()
        self.result_duskvis_label = ttk.Label(self.frame_middlewallop, textvariable=self.result_duskvis,
                                              width=7)
        self.result_duskvis_label.grid(row=2, column=1, sticky=tk.E, padx=10, pady=5)

        self.wind = tk.IntVar()
        self.wind5 = tk.Radiobutton(self.frame_middlewallop, text="0-5 kt", variable=self.wind, value=1)
        self.wind5.grid(row=5, column=0, padx=15, pady=15, sticky=tk.W)
        self.wind9 = tk.Radiobutton(self.frame_middlewallop, text="6-9 kt", variable=self.wind, value=2)
        self.wind9.grid(row=5, column=1, pady=15, sticky=tk.W)
        self.wind10 = tk.Radiobutton(self.frame_middlewallop, text=">9 kt", variable=self.wind, value=3)
        self.wind10.grid(row=5, column=2, pady=15, sticky=tk.W)
        self.wind5.select()

        self.maxvis_label = ttk.Label(self.frame_middlewallop, text="Daytime Maximum Vis (M)")
        self.maxvis_label.grid(sticky=tk.W, row=7, column=0, columnspan=4, padx=10, pady=5)
        self.maxvis_scale = tk.Scale(self.frame_middlewallop, from_=1000, to=50000, orient=tk.HORIZONTAL,
                                     length=220, resolution=1000)
        self.maxvis_scale.grid(row=6, column=0, columnspan=4, padx=10)
        self.maxvis_scale.set(25000)

        self.mintemp_label = ttk.Label(self.frame_middlewallop, text="Min Overnight Temp. (deg C)")
        self.mintemp_label.grid(sticky=tk.W, row=9, column=0, columnspan=4, padx=10, pady=5)
        self.mintemp_scale = tk.Scale(self.frame_middlewallop, from_=-20, to=25, orient=tk.HORIZONTAL, length=220)
        self.mintemp_scale.grid(row=8, column=0, columnspan=4, padx=10)

        self.dewpt_label = ttk.Label(self.frame_middlewallop, text="Dew Point at Max Vis (deg C)")
        self.dewpt_label.grid(sticky=tk.W, row=11, column=0, columnspan=4, padx=10, pady=5)
        self.dewpt_scale = tk.Scale(self.frame_middlewallop, from_=-20, to=25, orient=tk.HORIZONTAL, length=220)
        self.dewpt_scale.grid(row=10, column=0, columnspan=4, padx=10)


class WindGust:
    """Label window frame containing the widgets for calculating surface wind speed & gusts for various stability
        profiles."""

    def __init__(self, root):
        validcmd_int = (root.register(validate_int), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        self.frame_windgust = tk.LabelFrame(root, text='Wind Gust')
        self.frame_windgust.grid(row=2, column=0, columnspan=2, sticky=tk.NSEW, padx=5, pady=5)

        self.gradwind_label = ttk.Label(self.frame_windgust, text="Gradient Wind (kts):")
        self.gradwind_label.grid(sticky=tk.N, row=0, column=0, padx=5, pady=10)
        self.gradwind = tk.IntVar()
        self.gradwind_entry = ttk.Entry(self.frame_windgust, textvariable=self.gradwind, width=5, validate='key',
                                        validatecommand=validcmd_int)
        self.gradwind_entry.grid(row=0, column=1, pady=10, padx=5, sticky=tk.NW)
        self.gradwind.set(25)

        self.wind1200_label = ttk.Label(self.frame_windgust, text="1200ft Wind (kts) :")
        self.wind1200_label.grid(sticky=tk.S, row=2, column=0)
        self.result_wind1200 = tk.StringVar()
        self.result_wind1200_label = ttk.Label(self.frame_windgust, textvariable=self.result_wind1200, width=7)
        self.result_wind1200_label.grid(row=2, column=1, sticky=tk.S)

        self.calc_wind_button = ttk.Button(self.frame_windgust, text='Calculate')
        self.calc_wind_button.grid(row=1, column=0, sticky=tk.NE, columnspan=1)

        self.treeview = ttk.Treeview(self.frame_windgust, height=8)
        self.treeview.grid(row=0, column=2, rowspan=8, columnspan=1, padx=5, pady=5)

        self.treeview["columns"] = ("mean", "gust")
        self.treeview.column("mean", width=50)
        self.treeview.column("gust", width=50)
        self.treeview.heading("mean", text="Mean")
        self.treeview.heading("gust", text="Gust")

        self.treeview.insert("", 0, text="Steep inversion", values=())
        self.treeview.insert("", 1, text="Shallow inversion", values=())
        self.treeview.insert("", 2, text="SALR from surface", values=())
        self.treeview.insert("", 3, text="1-2 deg C / 1000ft", values=())
        self.treeview.insert("", 4, text="DALR from surface", values=())
        self.treeview.insert("", 5, text="DALR to 6000ft", values=())
        self.treeview.insert("", 6, text="Large CU / Steep LR", values=())

        self.author_label = ttk.Label(self.frame_windgust, text='v1.4.5 - mark.baker@metoffice.gov.uk')
        self.author_label.grid(row=14, column=0, columnspan=5, sticky=tk.W)


class StratusBase:
    """Label window frame containing the widgets for calculating stratus cloud base."""
    def __init__(self, root):
        validcmd_float = (root.register(validate_float_negative), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        validcmd_int = (root.register(validate_int), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')
        self.frame_stratus = tk.LabelFrame(root, text='Stratus Base')
        self.frame_stratus.grid(row=2, column=2, sticky=tk.NSEW, padx=5, pady=5)

        self.temp_label = ttk.Label(self.frame_stratus, text="Temp(C):")
        self.temp_label.grid(sticky=tk.W, row=0, column=0, padx=5, pady=5)
        self.temp = tk.DoubleVar()
        self.temp_entry = ttk.Entry(self.frame_stratus, textvariable=self.temp, width=5, validate='key',
                                    validatecommand=validcmd_float)
        self.temp_entry.grid(row=0, column=1, sticky=tk.W, padx=5, pady=5)
        self.temp.set(8.0)

        self.dewpt_label = ttk.Label(self.frame_stratus, text="DewPt.(C):")
        self.dewpt_label.grid(sticky=tk.W, row=0, column=2, padx=5, pady=5)
        self.dewpt = tk.DoubleVar()
        self.dewpt_entry = ttk.Entry(self.frame_stratus, textvariable=self.dewpt, width=5, validate='key',
                                     validatecommand=validcmd_float)
        self.dewpt_entry.grid(row=0, column=3, sticky=tk.W, padx=5, pady=5)
        self.dewpt.set(6.0)

        self.land_state = tk.IntVar()
        self.land_warming = tk.Radiobutton(self.frame_stratus, text="Land warming", variable=self.land_state, value=1)
        self.land_warming.grid(row=2, column=0, sticky=tk.W, columnspan=3, padx=5, pady=5)
        self.land_cooling = tk.Radiobutton(self.frame_stratus, text="Land cooling", variable=self.land_state, value=0)
        self.land_cooling.grid(row=2, column=2, sticky=tk.W, columnspan=3, padx=5, pady=5)
        self.land_cooling.select()

        self.calc_stratus_button = ttk.Button(self.frame_stratus, text='Calculate')
        self.calc_stratus_button.grid(row=3, column=0, columnspan=2, sticky=tk.W, padx=5, pady=5)

        self.stratusbase_label = ttk.Label(self.frame_stratus, text='BASE (ft):')
        self.stratusbase_label.grid(sticky=tk.E, row=3, column=2, padx=5, pady=5)
        self.result_stratusbase = tk.StringVar()
        self.result_stratusbase_label = ttk.Label(self.frame_stratus, textvariable=self.result_stratusbase, width=6)
        self.result_stratusbase_label.grid(row=3, column=3, sticky=tk.W, padx=5, pady=5)

        self.wind_label = ttk.Label(self.frame_stratus, text="Surface wind (V) (kts):")
        self.wind_label.grid(sticky=tk.E, row=4, column=0, columnspan=3, padx=5, pady=5)
        self.wind = tk.IntVar()
        self.wind_entry = ttk.Entry(self.frame_stratus, textvariable=self.wind, width=5, validate='key',
                                    validatecommand=validcmd_int)
        self.wind_entry.grid(row=4, column=3, sticky=tk.W, padx=5, pady=5)
        self.wind.set(10)

        self.air_state = tk.IntVar()
        self.dry_air = tk.Radiobutton(self.frame_stratus, text="Dry air (V*200)", variable=self.air_state, value=0)
        self.dry_air.grid(row=5, column=0, sticky=tk.W, columnspan=3, padx=5, pady=5)
        self.moist_air = tk.Radiobutton(self.frame_stratus, text="Moist air (V*75)", variable=self.air_state, value=1)
        self.moist_air.grid(row=5, column=2, sticky=tk.W, columnspan=3, padx=5, pady=5)
        self.moist_air.select()

        self.calc_stratus_button2 = ttk.Button(self.frame_stratus, text='Calculate')
        self.calc_stratus_button2.grid(row=6, column=0, columnspan=2, sticky=tk.W, padx=5, pady=5)

        self.stratusbase_label2 = ttk.Label(self.frame_stratus, text='BASE (ft):')
        self.stratusbase_label2.grid(sticky=tk.E, row=6, column=2, padx=5, pady=5)
        self.result_stratusbase2 = tk.StringVar()
        self.result_stratusbase2_label = ttk.Label(self.frame_stratus, textvariable=self.result_stratusbase2, width=6)
        self.result_stratusbase2_label.grid(row=6, column=3, sticky=tk.W, padx=5, pady=5)


def validate_float_negative(action, index, value_if_allowed,
                   prior_value, text, validation_type, trigger_type, widget_name):
    if not value_if_allowed:
        return True

    if text in '-1234567890':
        return True

    if text in '0123456789.-+':
        try:
            float(value_if_allowed)
            return True
        except ValueError:
            return False
    else:
        return False


def validate_float_positive(action, index, value_if_allowed,
                   prior_value, text, validation_type, trigger_type, widget_name):
    if not value_if_allowed:
        return True

    if text in '0123456789.+':
        try:
            float(value_if_allowed)
            return True
        except ValueError:
            return False
    else:
        return False


def validate_int(action, index, value_if_allowed,
                 prior_value, text, validation_type, trigger_type, widget_name):
    if not value_if_allowed:
        return True

    if text in '0123456789':
        try:
            int(value_if_allowed)
            return True
        except ValueError:
            return False
    else:
        return False
